//      t: Time, passed from PSIM by value
//   delt: Time step, passed from PSIM by value
//     in: input array, passed from PSIM by reference
//    out: output array, sent back to PSIM (Note: the values of out[*] can
//         be modified in PSIM)
// The maximum length of the input and output array "in" and "out" is 30.

// Las entradas del sistema son
// in[0]= ref;      referencia
// in[1]= sal;      variable a controlar
// in[2]= Fcpu;     frecuencia del oscilador de la uC
// in[3]= 0;
// in[4]= 0;
// in[5]= 0;
// in[6]= 0;

// Incluye librerias especiales o datos .h
//#include "dll.h"
//#include <windows.h>
//#include "ganancias_inversor.h"

// Coeficientes del controlador necesario:
// Ec. recursiva: u[k] = K*ev[k] + aK*ev[k-1];
#define		K		 10.653    //10.653
#define		aK		-10.00     //-10.00
// Registros internos del microcontrolador
#define		PER1	 156	   // Define el periodo del Timer1 (100ms).
#define		PRESC1	 32		   // Define el prescaler del Timer1.

// Para flags y dems:
#define		OFF		0			// Apagado.
#define		ON		1			// Encendido.

// PER1: Para timer de 16 bits, puede ser cualquier valor entero de 1 a 65535.
// PRESC1: Puede ser 1, 2, 4, 16, 32.
// Periodo de muestreo: Ts = PER1*PRESC1*Tosc
//
// Ejemplo: fs=100 Hz; Ts = 0.0100 seg, Fosc = 500kHz y PRESC1 = 32 => PER1 = Ts*Fosc/PRESC1 = 156,25.
// Se implementa un contador ascendente y descendente

// PROGRAMA QUE SE EJECUTA EN CADA PASO DE SIMULACION
void __declspec(dllexport) simuser (double t, double delt, double *in, double *out){
  	// variables reales
	static double y=0;   			// variable controlada
   	static double uk=0,  uc=0; 		// Accion de control
   	static double evk=0, evkm1=0;	// Error actual y anterior
   	static double ref;				// Referencia
	static double Ts;    			/* Periodo de muestreo */
	static double Fcpu, Tcpu; 		// Variables p/calculo de la relacin Ncont.
	
	// Variables enteras
	static int contador=0, Ncont, direccion = 1;
	static int PWM1, PWM2;
	static int ComparReg1, ComparReg2;
	
	// Variables booleanas:
	static char FLAG_calcu;				// Flag p/indicar si se calcula la acc. de control.
	static char FLAG_muestreo=ON;		// Flag p/indicar si se muestrea.
	
	// Se calcula el numero de conteos en un periodo: Valor mximo del contador
	//----------------------------------------------------------------------------------
	// CLCULOS INICIALES
	//----------------------------------------------------------------------------------
   	Fcpu = in[2];					// Adquiere frecuencia del oscilador					
	Tcpu = 1/Fcpu;					// y calcula periodo de la CPU.
	Ts = (PER1*PRESC1)/Fcpu;		// Calcula el periodo de muestreo
    Ncont=Ts/delt;                  // Calculo de Ncont
	
	// Se realiza el muestreo de las variables controladas
    if(FLAG_muestreo==ON){
       FLAG_muestreo=OFF;
        y = in[1];
        FLAG_calcu=ON;
    }
	
	// Calculo de la estrategia de control
	if(FLAG_calcu==ON){
		FLAG_calcu=OFF;
		ref = in[0]; 					// Adquiere la referencia (igual que generar adentro del uC)
		/*********************************************************************************/
		/*  Se calcula el error entre la salida de la planta y la referencia             */
		/*********************************************************************************/
		evk = ref - y;
		/*********************************************************************************/
		/*                   Calculo de la estrategia de control                         */
		/*********************************************************************************/
		uk = K*evk + aK*evkm1;	// Calcula acc. de control
		evkm1 = evk;				// Error de tensin anterior.
		//Para lazo abierto
		//uk = rd; 
		uc=(Ncont/4)*uk;			// Accin de control comparable con PWM
	} //termina el calculo de la estrategia de control

    //**********************************************************//
	// Incrementamos o decrementamos el contador segun corresponda
	contador = contador + direccion;
	if(contador == 0){
		direccion=1;
    	FLAG_muestreo=ON;
	}
	if(contador == Ncont/2){
		direccion=-1;
    	FLAG_muestreo=OFF;
	}
	if(contador == Ncont/2){ //((cont==0) || (cont==Ncont/2))
    	ComparReg1=  uc + Ncont/4;
    	ComparReg2= -uc + Ncont/4;
    }

	// Maquina de estado que define el patron PWM
    if(ComparReg1 > contador)
        PWM1=1;
    else
        PWM1=0;

    if(ComparReg2 > contador)
        PWM2=1;
    else
        PWM2=0;
    
	// Se actualizan los valores de las salidas
	out[0] = PWM1; //brazo 1
	out[1] = PWM2; //brazo 2
	out[2] = uk;
	out[3] = ref;
	out[4] = y;
	out[5] = evk;
	out[6] = contador;
	out[7] = uc;
}
